import ctypes
import threading
import os
import sys
from ctypes import windll, wintypes, CFUNCTYPE

# Constants from the Windows API
WHEEL_DELTA = 120
WM_MOUSEWHEEL = 0x020A
WM_MOUSEHWHEEL = 0x020E

# Mapping of WM_MOUSEWHEEL and WM_MOUSEHWHEEL to scroll vectors
SCROLL_BUTTONS = {
    WM_MOUSEWHEEL: (0, 1),
    WM_MOUSEHWHEEL: (1, 0)
}


class MSLLHOOKSTRUCT(ctypes.Structure):
    _fields_ = [
        ("pt", wintypes.POINT),
        ("mouseData", wintypes.DWORD),
        ("flags", wintypes.DWORD),
        ("time", wintypes.DWORD),
        ("dwExtraInfo", ctypes.POINTER(ctypes.c_ulong))
    ]


HOOKPROC = CFUNCTYPE(wintypes.LPARAM, ctypes.c_int, wintypes.WPARAM, ctypes.POINTER(MSLLHOOKSTRUCT))


class ScrollReader:
    def __init__(self):
        self._user32 = windll.user32
        self._k32 = windll.kernel32
        self._msg = wintypes.MSG()
        self._hooked = None
        self._mouse_hook = HOOKPROC(self._low_level_mouse_proc)
        self._thread = threading.Thread(target=self.run, args=())
        self._up_scroll_count = 0
        self._down_scroll_count = 0
        self._is_alive = True

    def run(self):
        try:
            self._set_hook()
            while self._is_alive and self._user32.GetMessageA(ctypes.byref(self._msg), 0, 0, 0) != 0:
                self._user32.TranslateMessage(ctypes.byref(self._msg))
                self._user32.DispatchMessageA(ctypes.byref(self._msg))
        except Exception as ex:
            print(f"Error occurred in thread: {ex}")

    def start(self):
        self._thread.start()

    def stop(self):
        self._is_alive = False
        self._user32.PostThreadMessageW(self._thread.ident, 0x0012, 0, 0)  # 0x0012 is WM_QUIT
        if self._thread.is_alive():
            self._thread.join()
        self._remove_hook()

    def get_scroll_counts(self):
        up_count = self._up_scroll_count
        down_count = self._down_scroll_count
        self._up_scroll_count = 0
        self._down_scroll_count = 0
        return up_count, down_count

    def _on_scroll(self, _, __, dx, dy):
        if dx == 0 and dy > 0:
            self._up_scroll_count += 1
        elif dx == 0 and dy < 0:
            self._down_scroll_count += 1

    def _get_module_handle(self):
        if hasattr(sys, "frozen") and sys.frozen in ("windows_exe", "console_exe"):  # Pyinstaller compatibility
            hMod = self._k32.GetModuleHandleW(None)
        else:
            hMod = self._k32.GetModuleHandleW(ctypes.c_wchar_p(os.path.abspath(__file__)))
        return hMod

    def _low_level_mouse_proc(self, nCode, wParam, lParam):
        if nCode == 0 and wParam in [WM_MOUSEWHEEL, WM_MOUSEHWHEEL]:
            mouse_struct = lParam.contents
            wm_mouse_scroll = wParam
            mx, my = SCROLL_BUTTONS[wm_mouse_scroll]
            dd = wintypes.SHORT(mouse_struct.mouseData >> 16).value // WHEEL_DELTA
            self._on_scroll(mouse_struct.pt.x, mouse_struct.pt.y, dd * mx, dd * my)
        return self._user32.CallNextHookEx(None, nCode, wParam, lParam)

    def _set_hook(self):
        self._hooked = self._user32.SetWindowsHookExW(14, self._mouse_hook, self._get_module_handle(), 0)
        if not self._hooked:
            print("Failed to install hook")
            sys.exit(1)

    def _remove_hook(self):
        if self._hooked is not None:
            self._user32.UnhookWindowsHookEx(self._hooked)
            self._hooked = None



import win32api
from ctypes import windll, Structure, c_long, byref
from win32con import VK_LBUTTON, VK_MBUTTON, VK_RBUTTON, VK_XBUTTON1, VK_XBUTTON2
from mistercar_input_devices.backend.windows.platform_specific.mouse.raw_mouse_reader import RawMouseReader


class POINT(Structure):
    _fields_ = [("x", c_long), ("y", c_long)]


MOUSE_BUTTONS = {
    "left": VK_LBUTTON,
    "middle": VK_MBUTTON,
    "right": VK_RBUTTON,
    "xbutton1": VK_XBUTTON1,
    "xbutton2": VK_XBUTTON2
}


class MouseReader:
    """Read cursor position, mouse movement, scrolling amount and button status"""
    def __init__(self):
        self._raw_mouse_reader = RawMouseReader()
        self._raw_mouse_reader.start()

    @staticmethod
    def get_cursor_position():
        pt = POINT()
        windll.user32.GetCursorPos(byref(pt))
        return pt.x, pt.y

    def get_relative_movement(self):
        return self._raw_mouse_reader.get_relative_movement()

    @staticmethod
    def get_button_states():
        return [int(win32api.GetAsyncKeyState(button_const) < 0) for button_name, button_const in MOUSE_BUTTONS.items()]

    def get_wheel(self):
        return self._raw_mouse_reader.get_wheel_delta()

    def get_horizontal_wheel(self):
        return self._raw_mouse_reader.get_hwheel_delta()

    def __del__(self):
        self._raw_mouse_reader.stop()



import platform


class MouseReader:
    def __init__(self):
        self.__operating_system = platform.system()
        if self.__operating_system == "Windows":
            from mistercar_input_devices.backend.windows.adapters.input_readers.mouse_reader_adapter import MouseReaderAdapter
            self.__reader = MouseReaderAdapter()
        elif self.__operating_system == "Linux":
            from mistercar_input_devices.backend.linux.adapters.input_readers.mouse_reader_adapter import MouseReaderAdapter
            self.__reader = MouseReaderAdapter()
        elif self.__operating_system == "Darwin":
            from mistercar_input_devices.backend.macos.adapters.input_readers.mouse_reader_adapter import MouseReaderAdapter
            self.__reader = MouseReaderAdapter()

    def get_cursor_position(self):
        return self.__reader.get_cursor_position()

    def get_relative_movement(self):
        return self.__reader.get_relative_movement()

    def get_button_states(self):
        return self.__reader.get_button_states()

    def get_wheel(self):
        return self.__reader.get_wheel()

    def get_horizontal_wheel(self):
        return self.__reader.get_horizontal_wheel()



import pywinusb.hid as hid
from typing import Optional, Dict, Tuple
from mistercar_input_devices.backend.multiplatform.input_readers.wheel_reader.base import WheelButton, WheelState


class TMXWheelAdapter:
    """Windows-specific adapter for Thrustmaster TMX wheel"""

    # Device identifiers
    VENDOR_ID = 0x044F  # Thrustmaster
    PRODUCT_ID = 0xB67F  # TMX Racing Wheel

    # Constants for pedal range
    MAX_PEDAL_VALUE = 0x3FF  # Maximum observed value (1023)

    def __init__(self):
        self._device = None
        self._last_data = None
        self._pedal_mode = "normal"  # or "swapped"
        self._initialize_device()

    def get_pedal_mode(self) -> str:
        """Get current pedal mode.

        Returns:
            str: Current pedal mode ("normal" or "swapped")
        """
        return self._pedal_mode

    def set_pedal_mode(self, mode: str) -> None:
        """Set pedal signal mapping to match the MODE LED on your wheel.

        The TMX wheel MODE button swaps the left and right pedal signals:

        Green LED (normal mode):
            Left pedal → Clutch signal
            Middle pedal → Brake signal
            Right pedal → Throttle signal

        Red LED (swapped mode):
            Left pedal → Throttle signal
            Middle pedal → Brake signal
            Right pedal → Clutch signal

        Use swapped mode if your pedals are physically reversed or you
        prefer a different arrangement.

        Important: Set this to match the actual LED color on your wheel!

        Args:
            mode: Either "normal" (green LED) or "swapped" (red LED)

        Raises:
            ValueError: If mode is not "normal" or "swapped"
        """
        if mode not in ("normal", "swapped"):
            raise ValueError('Pedal mode must be either "normal" or "swapped"')
        self._pedal_mode = mode

    def _initialize_device(self):
        """Initialize connection to the wheel"""
        all_devices = hid.HidDeviceFilter(
            vendor_id=self.VENDOR_ID,
            product_id=self.PRODUCT_ID
        ).get_devices()

        if not all_devices:
            raise RuntimeError("TMX wheel not found")

        self._device = all_devices[0]
        self._device.open()
        self._device.set_raw_data_handler(self._data_handler)

    def _data_handler(self, data):
        """Handle incoming data from the wheel"""
        self._last_data = bytes(data)

    def _parse_steering(self, data: bytes) -> float:
        """Parse steering value from raw data"""
        if not data or len(data) < 3:
            return 0.0

        # Combine two bytes for full steering range
        raw_value = (data[2] << 8) | data[1]
        # Convert to -1.0 to 1.0 range
        return (raw_value - 32767) / 32767

    def _parse_pedals(self, data: bytes) -> Tuple[float, float, float]:
        """Parse pedal values from raw data.

        Pedal byte mapping:
        - Bytes 3,4: Middle pedal (always Brake)
        - Bytes 5,6: Left pedal (Clutch in normal mode, Throttle in swapped mode)
        - Bytes 7,8: Right pedal (Throttle in normal mode, Clutch in swapped mode)

        Each pedal is a 16-bit little-endian value that starts high (~1023) when
        unpressed and decreases to 0 when fully pressed.

        Returns:
            Tuple[float, float, float]: (throttle, brake, clutch) values from 0.0 to 1.0
        """
        if not data or len(data) < 9:
            return 0.0, 0.0, 0.0

        # Brake is always middle pedal (bytes 3,4)
        brake_raw = (data[4] << 8) | data[3]

        # Left and right pedal signal assignment depends on mode
        if self._pedal_mode == "normal":
            # Green LED: Standard layout
            clutch_raw = (data[6] << 8) | data[5]    # bytes 5,6 = left pedal
            throttle_raw = (data[8] << 8) | data[7]  # bytes 7,8 = right pedal
        else:  # swapped mode
            # Red LED: Swapped layout
            throttle_raw = (data[6] << 8) | data[5]  # bytes 5,6 = left pedal
            clutch_raw = (data[8] << 8) | data[7]    # bytes 7,8 = right pedal

        # Convert to 0.0-1.0 range and invert (since raw values decrease when pressed)
        return (
            max(0.0, min(1.0, 1.0 - (throttle_raw / self.MAX_PEDAL_VALUE))),
            max(0.0, min(1.0, 1.0 - (brake_raw / self.MAX_PEDAL_VALUE))),
            max(0.0, min(1.0, 1.0 - (clutch_raw / self.MAX_PEDAL_VALUE)))
        )

    def _parse_buttons(self, data: bytes) -> Dict[WheelButton, bool]:
        """Parse button states from raw data"""
        if not data or len(data) < 15:
            return {}

        buttons = {}

        # Byte 11 buttons
        buttons[WheelButton.BUTTON_1] = bool(data[11] & 0x20)  # LB
        buttons[WheelButton.BUTTON_3] = bool(data[11] & 0x08)  # Left outer
        buttons[WheelButton.PADDLE_LEFT] = bool(data[11] & 0x01)  # Left paddle
        buttons[WheelButton.PADDLE_RIGHT] = bool(data[11] & 0x02)  # Right paddle
        buttons[WheelButton.XBOX_A] = bool(data[11] & 0x10)  # A button
        buttons[WheelButton.XBOX_X] = bool(data[11] & 0x04)  # X button
        buttons[WheelButton.VIEW] = bool(data[11] & 0x40)  # ⧉ button (formerly SELECT/BACK)
        buttons[WheelButton.MENU] = bool(data[11] & 0x80)  # ≡ button (formerly START)

        # Byte 12 buttons
        buttons[WheelButton.BUTTON_2] = bool(data[12] & 0x04)  # RB
        buttons[WheelButton.BUTTON_4] = bool(data[12] & 0x08)  # Right outer
        buttons[WheelButton.XBOX_B] = bool(data[12] & 0x01)  # B button
        buttons[WheelButton.XBOX_Y] = bool(data[12] & 0x02)  # Y button
        buttons[WheelButton.XBOX_GUIDE] = bool(data[12] & 0x10)  # Xbox/Power button

        # D-pad (byte 14)
        d_pad = data[14]

        # Clear all D-pad states first
        buttons[WheelButton.DPAD_UP] = False
        buttons[WheelButton.DPAD_DOWN] = False
        buttons[WheelButton.DPAD_LEFT] = False
        buttons[WheelButton.DPAD_RIGHT] = False

        # Set D-pad states based on value
        if d_pad == 0x00:  # UP
            buttons[WheelButton.DPAD_UP] = True
        elif d_pad == 0x01:  # UP-RIGHT
            buttons[WheelButton.DPAD_UP] = True
            buttons[WheelButton.DPAD_RIGHT] = True
        elif d_pad == 0x02:  # RIGHT
            buttons[WheelButton.DPAD_RIGHT] = True
        elif d_pad == 0x03:  # DOWN-RIGHT
            buttons[WheelButton.DPAD_DOWN] = True
            buttons[WheelButton.DPAD_RIGHT] = True
        elif d_pad == 0x04:  # DOWN
            buttons[WheelButton.DPAD_DOWN] = True
        elif d_pad == 0x05:  # DOWN-LEFT
            buttons[WheelButton.DPAD_DOWN] = True
            buttons[WheelButton.DPAD_LEFT] = True
        elif d_pad == 0x06:  # LEFT
            buttons[WheelButton.DPAD_LEFT] = True
        elif d_pad == 0x07:  # UP-LEFT
            buttons[WheelButton.DPAD_UP] = True
            buttons[WheelButton.DPAD_LEFT] = True

        return buttons

    def get_state(self) -> Optional[WheelState]:
        """Get current wheel state"""
        if not self._last_data:
            return None

        steering = self._parse_steering(self._last_data)
        throttle, brake, clutch = self._parse_pedals(self._last_data)
        buttons = self._parse_buttons(self._last_data)

        return WheelState(
            steering=steering,
            throttle=throttle,
            brake=brake,
            clutch=clutch,
            buttons=buttons
        )

    def close(self):
        """Close connection to the wheel"""
        if self._device:
            self._device.close()
            self._device = None



class GamepadEmulatorAdapter:
    def __init__(self):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    # === STICKS ===
    def emulate_left_stick(self, x, y):
        """Set left stick position"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_right_stick(self, x, y):
        """Set right stick position"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_left_stick_x(self, value):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_left_stick_y(self, value):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_right_stick_x(self, value):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_right_stick_y(self, value):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    # === TRIGGERS ===
    def emulate_triggers(self, left, right):
        """Set both trigger values"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_left_trigger(self, value):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_right_trigger(self, value):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    # === FACE BUTTONS ===
    def emulate_face_buttons(self, a, b, x, y):
        """Set all face button states"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_button_a(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_button_b(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_button_x(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_button_y(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    # === SHOULDER BUTTONS ===
    def emulate_shoulder_buttons(self, left, right):
        """Set both shoulder button states"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_left_shoulder(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_right_shoulder(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    # === THUMB BUTTONS ===
    def emulate_thumb_buttons(self, left, right):
        """Set both thumb button states"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_left_thumb(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_right_thumb(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    # === MENU BUTTONS ===
    def emulate_menu_buttons(self, back, start):
        """Set both menu button states"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_back_button(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_start_button(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    # === DPAD ===
    def emulate_dpad(self, up, down, left, right):
        """Set dpad state"""
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_dpad_up(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_dpad_down(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_dpad_left(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")

    def emulate_dpad_right(self, pressed):
        raise NotImplementedError("macOS gamepad emulation not yet implemented")



import platform


class KeyboardReader:
    def __init__(self):
        self.__operating_system = platform.system()
        if self.__operating_system == "Windows":
            from mistercar_input_devices.backend.windows.adapters.input_readers.keyboard_reader_adapter import KeyboardReaderAdapter
            self.__reader = KeyboardReaderAdapter()
        elif self.__operating_system == "Linux":
            from mistercar_input_devices.backend.linux.adapters.input_readers.keyboard_reader_adapter import KeyboardReaderAdapter
            self.__reader = KeyboardReaderAdapter()
        elif self.__operating_system == "Darwin":
            from mistercar_input_devices.backend.macos.adapters.input_readers.keyboard_reader_adapter import KeyboardReaderAdapter
            self.__reader = KeyboardReaderAdapter()

    def get_key_state(self, key):
        return self.__reader.get_key_state(key)

    def get_states_of_multiple_keys(self, keys_to_check):
        return self.__reader.get_states_of_multiple_keys(keys_to_check)



from typing import Optional
import pyautogui


class MouseEmulatorAdapter:
    def __init__(self):
        pass

    def move_mouse_by(self, dx: int, dy: int):
        pyautogui.moveRel(dx, dy)

    def move_cursor_to(self, x: int, y: int):
        pyautogui.moveTo(x, y)

    def scroll(self, clicks: int = 1):
        pyautogui.scroll(clicks * 120)  # Each click scrolls by 120 units

    def left_button_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._move_and_click(x, y, pyautogui.mouseDown, "left")

    def left_button_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._move_and_click(x, y, pyautogui.mouseUp, "left")

    def middle_button_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._move_and_click(x, y, pyautogui.mouseDown, "middle")

    def middle_button_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._move_and_click(x, y, pyautogui.mouseUp, "middle")

    def right_button_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._move_and_click(x, y, pyautogui.mouseDown, "right")

    def right_button_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._move_and_click(x, y, pyautogui.mouseUp, "right")

    def xbutton1_press(self, x: Optional[int] = None, y: Optional[int] = None):
        raise NotImplementedError

    def xbutton1_release(self, x: Optional[int] = None, y: Optional[int] = None):
        raise NotImplementedError

    def xbutton2_press(self, x: Optional[int] = None, y: Optional[int] = None):
        raise NotImplementedError

    def xbutton2_release(self, x: Optional[int] = None, y: Optional[int] = None):
        raise NotImplementedError

    def click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self.left_click(x, y, duration)

    def left_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self._move_and_click(x, y, pyautogui.click, "left", duration)

    def middle_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self._move_and_click(x, y, pyautogui.click, "middle", duration)

    def right_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self._move_and_click(x, y, pyautogui.click, "right", duration)

    def xbutton1_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        raise NotImplementedError

    def xbutton2_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        raise NotImplementedError

    def _move_and_click(self, x, y, action, button, duration=0.0):
        if x is not None and y is not None:
            pyautogui.moveTo(x, y)
        action(button=button, duration=duration)



import evdev
from typing import List, Optional, Callable
from mistercar_input_devices.device_analysis.analyzers.base import DeviceAnalyzer, DeviceInfo


class EvdevAnalyzer(DeviceAnalyzer):
    """Linux device analyzer using evdev."""

    def __init__(self):
        super().__init__()
        self._device: Optional[evdev.InputDevice] = None
        self._running = False

    def list_devices(self) -> List[DeviceInfo]:
        """List all available input devices.

        Returns:
            List[DeviceInfo]: List of available devices

        Raises:
            PermissionError: If user doesn't have permission to access input devices
        """
        devices = []
        try:
            for path in evdev.list_devices():
                try:
                    device = evdev.InputDevice(path)
                    # Extract vendor and product ID from device info
                    vendor_id = device.info.vendor
                    product_id = device.info.product

                    devices.append(DeviceInfo(
                        vendor_id=vendor_id,
                        product_id=product_id,
                        name=device.name,
                        description=f"Path: {device.path}",
                        path=device.path
                    ))
                except (PermissionError, OSError) as e:
                    # Skip devices we can't access
                    continue
        except Exception as e:
            raise RuntimeError(f"Failed to list devices: {e}")

        return devices

    def open_device(self, vendor_id: int, product_id: int) -> bool:
        """Open a specific device for analysis.

        Args:
            vendor_id: USB vendor ID of the device
            product_id: USB product ID of the device

        Returns:
            bool: True if device was opened successfully

        Raises:
            PermissionError: If user doesn't have permission to access the device
            RuntimeError: If device cannot be opened
        """
        self.close_device()

        try:
            # Find device with matching vendor/product ID
            for path in evdev.list_devices():
                device = evdev.InputDevice(path)
                if (device.info.vendor == vendor_id and
                        device.info.product == product_id):
                    self._device = device
                    return True
        except Exception as e:
            raise RuntimeError(f"Failed to open device: {e}")

        return False

    def close_device(self) -> None:
        """Close the currently opened device."""
        if self._device:
            self._device.close()
            self._device = None

    def read_data(self) -> Optional[bytes]:
        """Read a single data packet from the device.

        Returns:
            Optional[bytes]: Raw input data or None if no data available

        Note:
            evdev doesn't provide raw HID data like Windows/macOS.
            This implementation converts evdev events to a standardized format.
        """
        if not self._device:
            return None

        try:
            events = self._device.read()
            if events:
                # Convert events to standardized binary format
                # This is a placeholder - actual implementation would depend
                # on how we want to standardize the data across platforms
                return bytes([event.type, event.code, event.value & 0xFF]
                             for event in events)
        except BlockingIOError:
            # No data available
            pass
        except Exception as e:
            raise RuntimeError(f"Error reading device: {e}")

        return None

    def _monitor_loop(self) -> None:
        """Internal monitoring loop.

        Note:
            Overridden to use evdev's async capabilities if needed.
        """
        try:
            while self._running:
                data = self.read_data()
                if data and self.data_callback:
                    self.data_callback(data)
        except Exception as e:
            print(f"Error in monitor loop: {e}")
            self._running = False



from mistercar_input_devices.backend.windows.platform_specific.keys_and_mouse import KeysAndMouse, KEY_RELEASE


class KeyboardEmulatorAdapter:
    def __init__(self):
        self._keys_and_mouse = KeysAndMouse()

    def emulate_key(self, key, value):
        if value == 0:
            self._keys_and_mouse.direct_key(key, KEY_RELEASE)
        else:
            self._keys_and_mouse.direct_key(key)

    def emulate_multiple_keys(self, keys_list):
        for key in keys_list:
            self._keys_and_mouse.direct_key(key)



from mistercar_input_devices.backend.windows.platform_specific.pyxinput import vController


class GamepadEmulatorAdapter:
    def __init__(self):
        self._gamepad = vController()

    # === STICKS ===
    def emulate_left_stick(self, x, y):
        """Set left stick position"""
        self._gamepad.set_value("AxisLx", x)
        self._gamepad.set_value("AxisLy", y)

    def emulate_right_stick(self, x, y):
        """Set right stick position"""
        self._gamepad.set_value("AxisRx", x)
        self._gamepad.set_value("AxisRy", y)

    def emulate_left_stick_x(self, value):
        self._gamepad.set_value("AxisLx", value)

    def emulate_left_stick_y(self, value):
        self._gamepad.set_value("AxisLy", value)

    def emulate_right_stick_x(self, value):
        self._gamepad.set_value("AxisRx", value)

    def emulate_right_stick_y(self, value):
        self._gamepad.set_value("AxisRy", value)

    # === TRIGGERS ===
    def emulate_triggers(self, left, right):
        """Set both trigger values"""
        self._gamepad.set_value("TriggerL", left)
        self._gamepad.set_value("TriggerR", right)

    def emulate_left_trigger(self, value):
        self._gamepad.set_value("TriggerL", value)

    def emulate_right_trigger(self, value):
        self._gamepad.set_value("TriggerR", value)

    # === FACE BUTTONS ===
    def emulate_face_buttons(self, a, b, x, y):
        """Set all face button states"""
        self._gamepad.set_value("BtnA", 1 if a else 0)
        self._gamepad.set_value("BtnB", 1 if b else 0)
        self._gamepad.set_value("BtnX", 1 if x else 0)
        self._gamepad.set_value("BtnY", 1 if y else 0)

    def emulate_button_a(self, pressed):
        self._gamepad.set_value("BtnA", 1 if pressed else 0)

    def emulate_button_b(self, pressed):
        self._gamepad.set_value("BtnB", 1 if pressed else 0)

    def emulate_button_x(self, pressed):
        self._gamepad.set_value("BtnX", 1 if pressed else 0)

    def emulate_button_y(self, pressed):
        self._gamepad.set_value("BtnY", 1 if pressed else 0)

    # === SHOULDER BUTTONS ===
    def emulate_shoulder_buttons(self, left, right):
        """Set both shoulder button states"""
        self._gamepad.set_value("BtnShoulderL", 1 if left else 0)
        self._gamepad.set_value("BtnShoulderR", 1 if right else 0)

    def emulate_left_shoulder(self, pressed):
        self._gamepad.set_value("BtnShoulderL", 1 if pressed else 0)

    def emulate_right_shoulder(self, pressed):
        self._gamepad.set_value("BtnShoulderR", 1 if pressed else 0)

    # === THUMB BUTTONS ===
    def emulate_thumb_buttons(self, left, right):
        """Set both thumb button states"""
        self._gamepad.set_value("BtnThumbL", 1 if left else 0)
        self._gamepad.set_value("BtnThumbR", 1 if right else 0)

    def emulate_left_thumb(self, pressed):
        self._gamepad.set_value("BtnThumbL", 1 if pressed else 0)

    def emulate_right_thumb(self, pressed):
        self._gamepad.set_value("BtnThumbR", 1 if pressed else 0)

    # === MENU BUTTONS ===
    def emulate_menu_buttons(self, back, start):
        """Set both menu button states"""
        self._gamepad.set_value("BtnBack", 1 if back else 0)
        self._gamepad.set_value("BtnStart", 1 if start else 0)

    def emulate_back_button(self, pressed):
        self._gamepad.set_value("BtnBack", 1 if pressed else 0)

    def emulate_start_button(self, pressed):
        self._gamepad.set_value("BtnStart", 1 if pressed else 0)

    # === DPAD ===
    def emulate_dpad(self, up, down, left, right):
        """Set dpad state - uses original Dpad control with combined value"""
        value = 0
        if up: value |= 1
        if down: value |= 2
        if left: value |= 4
        if right: value |= 8
        self._gamepad.set_value("Dpad", value)

    def emulate_dpad_up(self, pressed):
        self._gamepad.set_value("Dpad", 1 if pressed else 0)

    def emulate_dpad_down(self, pressed):
        self._gamepad.set_value("Dpad", 2 if pressed else 0)

    def emulate_dpad_left(self, pressed):
        self._gamepad.set_value("Dpad", 4 if pressed else 0)

    def emulate_dpad_right(self, pressed):
        self._gamepad.set_value("Dpad", 8 if pressed else 0)



# Code from https://github.com/Sentdex/pygta5/blob/master/original_project/keys.py
# Further modifications by PypayaTech

import ctypes
import time
from threading import Thread
from time import sleep
from queue import Queue
from typing import Optional
from win32con import WHEEL_DELTA
from mistercar_input_devices.backend.windows.platform_specific.mouse.mouse_input_constants import MOUSEEVENTF_MOVE, \
    MOUSEEVENTF_ABSOLUTE, MOUSEEVENTF_WHEEL, MOUSEEVENTF_LEFTDOWN, MOUSEEVENTF_LEFTUP, MOUSEEVENTF_RIGHTDOWN, \
    MOUSEEVENTF_RIGHTUP, MOUSEEVENTF_MIDDLEDOWN, MOUSEEVENTF_MIDDLEUP, MOUSEEVENTF_XDOWN, MOUSEEVENTF_XUP
from mistercar_input_devices.backend.windows.platform_specific.structures import MOUSEINPUT, KEYBDINPUT, HARDWAREINPUT, \
    _INPUTunion, INPUT
from mistercar_input_devices.backend.windows.platform_specific.keyboard.virtual_keys import vk
from mistercar_input_devices.backend.windows.platform_specific.keyboard.direct_keys import dk


DIRECT_KEYS = 0x0008
VIRTUAL_KEYS = 0x0000
KEY_PRESS = 0x0000
KEY_RELEASE = 0x0002

XBUTTON1 = 0x0001
XBUTTON2 = 0x0002


# main keys and mouse class
class KeysAndMouse(object):
    common = None
    standalone = False

    # instance of worker class
    keys_and_mouse_worker = None
    keys_process = None

    # setup object
    def __init__(self, common=None):
        self.keys_and_mouse_worker = KeysAndMouseWorker(self)
        # Thread(target=self.keys_worker.processQueue).start()
        self.common = common
        if common is None:
            self.standalone = True
        self._screen_width = ctypes.windll.user32.GetSystemMetrics(0)
        self._screen_height = ctypes.windll.user32.GetSystemMetrics(1)

    # parses keys string and adds keys to the queue
    def parseKeyString(self, string):

        # print keys
        if not self.standalone:
            self.common.info("Processing keys: %s" % string)

        key_queue = []
        errors = []

        # defaults to direct keys
        key_type = DIRECT_KEYS

        # split by comma
        keys = string.upper().split(",")

        # translate
        for key in keys:

            # up, down or stroke?
            up = True
            down = True
            direction = key.split("_")
            subkey = direction[0]
            if len(direction) >= 2:
                if direction[1] == 'UP':
                    down = False
                else:
                    up = False

            # switch to virtual keys
            if subkey == "VK":
                key_type = VIRTUAL_KEYS

            # switch to direct keys
            elif subkey == "DK":
                key_type = DIRECT_KEYS

            # key code
            elif subkey.startswith("0x"):
                subkey = int(subkey, 16)
                if subkey > 0 and subkey < 256:
                    key_queue.append({
                        "key": int(subkey),
                        "okey": subkey,
                        "time": 0,
                        "up": up,
                        "down": down,
                        "type": key_type,
                    })
                else:
                    errors.append(key)

            # pause
            elif subkey.startswith("-"):
                time = float(subkey.replace("-", "")) / 1000
                if time > 0 and time <= 10:
                    key_queue.append({
                        "key": None,
                        "okey": "",
                        "time": time,
                        "up": False,
                        "down": False,
                        "type": None,
                    })
                else:
                    errors.append(key)

            # direct key
            elif key_type == DIRECT_KEYS and subkey in dk:
                key_queue.append({
                    "key": dk[subkey],
                    "okey": subkey,
                    "time": 0,
                    "up": up,
                    "down": down,
                    "type": key_type,
                })

            # virtual key
            elif key_type == VIRTUAL_KEYS and subkey in vk:
                key_queue.append({
                    "key": vk[subkey],
                    "okey": subkey,
                    "time": 0,
                    "up": up,
                    "down": down,
                    "type": key_type,
                })

            # no match?
            else:
                errors.append(key)

        # if there are errors, do not process keys
        if len(errors):
            return errors

        # create new thread if there is no active one
        if self.keys_process is None or not self.keys_process.isAlive():
            self.keys_process = Thread(target=self.keys_and_mouse_worker.processQueue)
            self.keys_process.start()

        # add keys to queue
        for i in key_queue:
            self.keys_and_mouse_worker.key_queue.put(i)
        self.keys_and_mouse_worker.key_queue.put(None)

        return True

    # direct key press
    def direct_key(self, key, direction=None, type=None):
        if type is None:
            type = DIRECT_KEYS
        if direction is None:
            direction = KEY_PRESS
        if key.startswith("0x"):
            key = int(key, 16)
        else:
            key = key.upper()
            lookup_table = dk if type == DIRECT_KEYS else vk
            key = lookup_table[key] if key in lookup_table else 0x0000

        self.keys_and_mouse_worker.sendKey(key, direction | type)

    def _mouse_press(self, event_type: int, x: Optional[int] = None, y: Optional[int] = None, data=0):
        flags = event_type
        if x is not None and y is not None:
            flags |= MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE
            x, y = self._normalize_coords(x, y)
        else:
            x, y = 0, 0
        self.keys_and_mouse_worker.sendMouse(x, y, data, flags)

    def move_mouse_by(self, dx: int, dy: int):
        self.keys_and_mouse_worker.sendMouse(dx, dy, 0, MOUSEEVENTF_MOVE)

    def move_cursor_to(self, x, y):
        x, y = self._normalize_coords(x, y)
        self.keys_and_mouse_worker.sendMouse(x, y, 0, MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE)

    def scroll(self, clicks: int = 1):
        self.keys_and_mouse_worker.sendMouse(0, 0, clicks * WHEEL_DELTA, MOUSEEVENTF_WHEEL)

    def left_button_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_LEFTDOWN, x, y)

    def left_button_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_LEFTUP, x, y)

    def middle_button_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_MIDDLEDOWN, x, y)

    def middle_button_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_MIDDLEUP, x, y)

    def right_button_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_RIGHTDOWN, x, y)

    def right_button_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_RIGHTUP, x, y)

    def xbutton1_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_XDOWN, x, y, XBUTTON1)

    def xbutton1_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_XUP, x, y, XBUTTON1)

    def xbutton2_press(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_XDOWN, x, y, XBUTTON2)

    def xbutton2_release(self, x: Optional[int] = None, y: Optional[int] = None):
        self._mouse_press(MOUSEEVENTF_XUP, x, y, XBUTTON2)

    def click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self.left_button_press(x, y)
        time.sleep(duration)
        self.left_button_release(x, y)

    def left_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self.click(x, y, duration)

    def middle_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self.middle_button_press(x, y)
        time.sleep(duration)
        self.middle_button_release(x, y)

    def right_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self.right_button_press(x, y)
        time.sleep(duration)
        self.right_button_release(x, y)

    def xbutton1_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self.xbutton1_press(x, y)
        time.sleep(duration)
        self.xbutton1_release(x, y)

    def xbutton2_click(self, x: Optional[int] = None, y: Optional[int] = None, duration: float = 0.0):
        self.xbutton2_press(x, y)
        time.sleep(duration)
        self.xbutton2_release(x, y)

    def _normalize_coords(self, x, y):
        return (x * 65535) // self._screen_width, (y * 65535) // self._screen_height


# threaded sending keys class
class KeysAndMouseWorker:
    # keys and mouse object
    keys_and_mouse = None

    # queue of keys
    key_queue = Queue()

    # init
    def __init__(self, keys_and_mouse):
        self.keys_and_mouse = keys_and_mouse

    # main function, process key's queue in loop
    def processQueue(self):

        # endless loop
        while True:

            # get one key
            key = self.key_queue.get()

            # terminate process if queue is empty
            if key is None:
                self.key_queue.task_done()
                if self.key_queue.empty():
                    return
                continue
            # print key
            elif not self.keys_and_mouse.standalone:
                self.keys_and_mouse.common.info(
                    "Key: \033[1;35m%s/%s\033[0;37m, duration: \033[1;35m%f\033[0;37m, direction: \033[1;35m%s\033[0;37m, type: \033[1;35m%s" % (
                        key["okey"] if key["okey"] else "None",
                        key["key"], key["time"],
                        "UP" if key["up"] and not key["down"] else "DOWN" if not key["up"] and key[
                            "down"] else "BOTH" if key["up"] and key["down"] else "NONE",
                        "None" if key["type"] is None else "DK" if key["type"] == DIRECT_KEYS else "VK"),
                    "\033[0;35mKEY:    \033[0;37m"
                    )

            # if it's a key
            if key["key"]:

                # press
                if key["down"]:
                    self.sendKey(key["key"], KEY_PRESS | key["type"])

                # wait
                sleep(key["time"])

                # and release
                if key["up"]:
                    self.sendKey(key["key"], KEY_RELEASE | key["type"])

            # not an actual key, just pause
            else:
                sleep(key["time"])

            # mark as done (decrement internal queue counter)
            self.key_queue.task_done()

    # send key
    def sendKey(self, key, type):
        self.SendInput(self.Keyboard(key, type))

    # send mouse
    def sendMouse(self, dx, dy, data, flags):
        self.SendInput(self.Mouse(flags, dx, dy, data))

    # send input
    def SendInput(self, *inputs):
        nInputs = len(inputs)
        LPINPUT = INPUT * nInputs
        pInputs = LPINPUT(*inputs)
        cbSize = ctypes.c_int(ctypes.sizeof(INPUT))
        return ctypes.windll.user32.SendInput(nInputs, pInputs, cbSize)

    # get input object
    def Input(self, structure):
        if isinstance(structure, MOUSEINPUT):
            return INPUT(0, _INPUTunion(mi=structure))
        if isinstance(structure, KEYBDINPUT):
            return INPUT(1, _INPUTunion(ki=structure))
        if isinstance(structure, HARDWAREINPUT):
            return INPUT(2, _INPUTunion(hi=structure))
        raise TypeError('Cannot create INPUT structure!')

    # mouse input
    def MouseInput(self, x, y, data, flags):
        return MOUSEINPUT(x, y, data, flags, 0, None)

    # keyboard input
    def KeybdInput(self, code, flags):
        return KEYBDINPUT(code, code, flags, 0, None)

    # hardware input
    def HardwareInput(self, message, parameter):
        return HARDWAREINPUT(message & 0xFFFFFFFF,
                             parameter & 0xFFFF,
                             parameter >> 16 & 0xFFFF)

    # mouse object
    def Mouse(self, flags, x=0, y=0, data=0):
        return self.Input(self.MouseInput(x, y, data, flags))

    # keyboard object
    def Keyboard(self, code, flags=0):
        return self.Input(self.KeybdInput(code, flags))

    # hardware object
    def Hardware(self, message, parameter=0):
        return self.Input(self.HardwareInput(message, parameter))


def main():
    keys = KeysAndMouse()

    sleep_time = 2
    x, y = 500, 200

    sleep(sleep_time)

    # Left click at current point
    print("Left click at current point")
    keys.left_click()
    sleep(sleep_time)

    # Left click at specific point
    print(f"Left click at x={x}, y={y}")
    keys.left_click(x=x, y=y)
    sleep(sleep_time)

    # Middle click at current point
    print("Middle click at current point")
    keys.middle_click()
    sleep(sleep_time)

    # Middle click at specific point
    print(f"Middle click at x={x}, y={y}")
    keys.middle_click(x=x, y=y)
    sleep(sleep_time)

    # Right click at current point
    print("Right click at current point")
    keys.right_click()
    sleep(sleep_time)

    # Right click at specific point
    print(f"Right click at x={x}, y={y}")
    keys.right_click(x=x, y=y)
    sleep(sleep_time)

    # Click XBUTTON1
    print("Click XBUTTON1")
    keys.xbutton1_click()
    sleep(sleep_time)

    # Click XBUTTON2
    print("Click XBUTTON2")
    keys.xbutton2_click()
    sleep(sleep_time)

    # Moving
    print("Moving mouse by dx = 50, dy = 50")
    keys.move_mouse_by(dx=50, dy=50)
    sleep(sleep_time)

    # Teleporting
    print(f"Teleporting cursor to x={x}, y={y}")
    keys.move_cursor_to(x=x, y=y)
    sleep(sleep_time)

    # Scrolling
    print("Scrolling mouse 3 times up")
    keys.scroll(clicks=3)
    sleep(sleep_time)

    # keyboard (direct keys)
    keys.direct_key("a")
    sleep(0.04)
    keys.direct_key("a", KEY_RELEASE)

    # keyboard (virtual keys)
    keys.direct_key("a", type=VIRTUAL_KEYS)
    sleep(0.04)
    keys.direct_key("a", KEY_RELEASE, VIRTUAL_KEYS)

    # queue of keys (direct keys, threaded, only for keybord input)
    keys.parseKeyString("a_down,-4,a_up,0x01")  # -4 - pause for 4 ms, 0x00 - hex code of Esc

    # queue of keys (virtual keys, threaded, only for keybord input)
    keys.parseKeyString("vk,a_down,-4,a_up")  # -4 - pause for 4 ms


if __name__ == "__main__":
    main()



"""Virtual Controller Object for Python

Python Implepentation of vXbox from:
http://vjoystick.sourceforge.net/site/index.php/vxbox"""
from ctypes import *
import os
import platform

if '32' in platform.architecture()[0]:
    arc = '86'
else:
    arc = '64'

_path = os.path.abspath(os.path.join(
    os.path.dirname(__file__),
    'vXboxInterface-x{}'.format(arc),
    'vXboxInterface.dll'
))
_xinput = WinDLL(_path)


class MaxInputsReachedError(Exception):
    """Exception when no inputs are available.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message


class vController(object):
    """Virtual Controller Object
    percent, Bool, Determines if absolute or percentage values are passed"""
    DPAD_OFF = 0
    DPAD_UP = 1
    DPAD_DOWN = 2
    DPAD_LEFT = 4
    DPAD_RIGHT = 8

    def __init__(self, percent=True):
        self.percent = percent
        self.id = 0
        self.PlugIn()

    def __del__(self):
        """Unplug self if object is cleaned up"""
        self.UnPlug()

    @classmethod
    def available_ids(self):
        ids = [
            x for x in range(1, 5)
            if _xinput.isControllerExists(c_int(x)) == 0
        ]

        return ids

    def PlugIn(self):
        """Take next available controller id and plug in to Virtual USB Bus"""
        ids = self.available_ids()
        if len(ids) == 0:
            raise MaxInputsReachedError('Max Inputs Reached')

        self.id = ids[0]

        _xinput.PlugIn(self.id)
        while self.id in self.available_ids():
            pass

    def UnPlug(self, force=False):
        """Unplug controller from Virtual USB Bus and free up ID"""
        if force:
            _xinput.UnPlugForce(c_uint(self.id))
        else:
            _xinput.UnPlug(c_uint(self.id))
        while self.id not in self.available_ids():
            if self.id == 0:
                break

    def set_value(self, control, value=None):
        """Set a value on the controller
    If percent is True all controls will accept a value between -1.0 and 1.0

    If not then:
        Triggers are 0 to 255
        Axis are -32768 to 32767

    Control List:
        AxisLx          , Left Stick X-Axis
        AxisLy          , Left Stick Y-Axis
        AxisRx          , Right Stick X-Axis
        AxisRy          , Right Stick Y-Axis
        BtnBack         , Menu/Back Button
        BtnStart        , Start Button
        BtnA            , A Button
        BtnB            , B Button
        BtnX            , X Button
        BtnY            , Y Button
        BtnThumbL       , Left Thumbstick Click
        BtnThumbR       , Right Thumbstick Click
        BtnShoulderL    , Left Shoulder Button
        BtnShoulderR    , Right Shoulder Button
        Dpad            , Set Dpad Value (0 = Off, Use DPAD_### Constants)
        TriggerL        , Left Trigger
        TriggerR        , Right Trigger

    """
        func = getattr(_xinput, 'Set' + control)

        if 'Axis' in control:
            target_type = c_short

            if self.percent:
                target_value = int(32767 * value)
            else:
                target_value = value
        elif 'Btn' in control:
            target_type = c_bool
            target_value = bool(value)
        elif 'Trigger' in control:
            target_type = c_byte

            if self.percent:
                target_value = int(255 * value)
            else:
                target_value = value
        elif 'Dpad' in control:
            target_type = c_int
            target_value = int(value)

        func(c_uint(self.id), target_type(target_value))


def main():
    import time
    cons = []

    print('Testings multiple connections')
    while True:
        print('Connecting Controller:')
        try:
            cons.append(vController())
        except MaxInputsReachedError:
            break
        else:
            print('Available:', vController.available_ids())
            print('This ID:', cons[-1].id)

        # time.sleep(1)

    print('Done, disconnecting controllers.')
    del cons
    print('Available:', vController.available_ids())
    time.sleep(2)

    print('Testing Value setting')
    print('Connecting Controller:')
    try:
        con = vController()
    except MaxInputsReachedError:
        print('Unable to connect controller for testing.')
    else:
        print('This ID:', con.id)
        print('Available:', vController.available_ids())
        print('Setting TriggerR and AxisLx:')
        for x in range(11):
            val = x / 10
            print(val)
            con.set_value('TriggerR', val)
            con.set_value('AxisLx', val)
            time.sleep(0.5)

        print('Done, disconnecting controller.')
        del con
        print('Available:', vController.available_ids())
        time.sleep(2)


if __name__ == '__main__':
    main()



import pyautogui


class KeyboardEmulatorAdapter:
    def __init__(self):
        pass

    def emulate_key(self, key, value):
        if value == 1:
            pyautogui.keyDown(key)
        elif value == 0:
            pyautogui.keyUp(key)

    def emulate_multiple_keys(self, keys_list):
        for key in keys_list:
            self.emulate_key(key, 1)



MOUSEEVENTF_MOVE = 0x0001  # Movement occurred.

MOUSEEVENTF_LEFTDOWN = 0x0002  # The left button was pressed.

MOUSEEVENTF_LEFTUP = 0x0004  # The left button was released.

MOUSEEVENTF_RIGHTDOWN = 0x0008  # The right button was pressed.

MOUSEEVENTF_RIGHTUP = 0x0010  # The right button was released.

MOUSEEVENTF_MIDDLEDOWN = 0x0020  # The middle button was pressed.

MOUSEEVENTF_MIDDLEUP = 0x0040  # The middle button was released.

MOUSEEVENTF_XDOWN = 0x0080  # An X button was pressed.

MOUSEEVENTF_XUP = 0x0100  # An X button was released.

MOUSEEVENTF_WHEEL = 0x0800  # The wheel was moved, if the mouse has a wheel.
# The amount of movement is specified in mouseData.

MOUSEEVENTF_HWHEEL = 0x1000  # The wheel was moved horizontally, if the mouse has a wheel.
# The amount of movement is specified in mouseData.
# Windows XP/2000: This value is not supported.

MOUSEEVENTF_MOVE_NOCOALESCE = 0x2000  # The WM_MOUSEMOVE messages will not be coalesced.
# The default behavior is to coalesce WM_MOUSEMOVE messages.
# Windows XP/2000: This value is not supported.

MOUSEEVENTF_VIRTUALDESK = 0x4000  # Maps coordinates to the entire desktop.
# Must be used with MOUSEEVENTF_ABSOLUTE.

MOUSEEVENTF_ABSOLUTE = 0x8000
# The dx and dy members contain normalized absolute coordinates.
# If the flag is not set, dx and dy contain relative data (the change in position since the last reported position).
# This flag can be set, or not set, regardless of what kind of mouse or other pointing device, if any,
# is connected to the system. For further information about relative mouse motion, see the following Remarks section:
# https://learn.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-mouseinput#remarks



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional, List, Callable, Any, Dict
import threading
import time


@dataclass
class DeviceInfo:
    """Basic information about a USB device"""
    vendor_id: int
    product_id: int
    name: str
    description: str = ""
    path: str = ""

    def __str__(self):
        return f"{self.name} (VID: 0x{self.vendor_id:04x}, PID: 0x{self.product_id:04x})"


class DeviceAnalyzer(ABC):
    """Base class for device analyzers"""

    def __init__(self):
        self.data_callback: Optional[Callable[[bytes], None]] = None
        self.device_info: Optional[DeviceInfo] = None
        self._monitoring_thread: Optional[threading.Thread] = None
        self._running = False

    @abstractmethod
    def list_devices(self) -> List[DeviceInfo]:
        """List all available devices"""
        pass

    @abstractmethod
    def open_device(self, vendor_id: int, product_id: int) -> bool:
        """Open a specific device for analysis"""
        pass

    @abstractmethod
    def close_device(self) -> None:
        """Close the currently opened device"""
        pass

    @abstractmethod
    def read_data(self) -> Optional[bytes]:
        """Read a single data packet from the device
        Returns None if no data is available"""
        pass

    def start_monitoring(self, callback: Callable[[bytes], None]) -> None:
        """Start monitoring device data with a callback

        callback will be called with each data packet received
        """
        if self._monitoring_thread is not None:
            raise RuntimeError("Already monitoring")

        self.data_callback = callback
        self._running = True
        self._monitoring_thread = threading.Thread(target=self._monitor_loop)
        self._monitoring_thread.daemon = True
        self._monitoring_thread.start()

    def stop_monitoring(self) -> None:
        """Stop monitoring device data"""
        self._running = False
        if self._monitoring_thread:
            self._monitoring_thread.join()
            self._monitoring_thread = None

    def _monitor_loop(self) -> None:
        """Internal monitoring loop"""
        while self._running:
            data = self.read_data()
            if data and self.data_callback:
                self.data_callback(data)
            time.sleep(0.001)  # Prevent 100% CPU usage
